import socket

def get_local_ip():
    ip = '<host-name>'
    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        s.connect(("8.8.8.8", 53))
        ip = s.getsockname()[0]
        s.close()
    except:
        ip = socket.gethostname()
    return ip

if __name__ == '__main__':
    print(get_local_ip())


import logging
import threading
import time

# Credits to https://gist.github.com/cypreess/5481681


class PeriodicThread(object):
    """
    Python periodic Thread using Timer with instant cancellation
    """

    def __init__(self, callback=None, period=1, name=None, *args, **kwargs):
        self.name = name
        self.args = args
        self.kwargs = kwargs
        self.callback = callback
        self.period = period
        self.stop = False
        self.current_timer = None
        self.schedule_lock = threading.Lock()

    def start(self):
        """
        Mimics Thread standard start method
        """
        self.schedule_timer()

    def run(self):
        """
        By default run callback. Override it if you want to use inheritance
        """
        if self.callback is not None:
            self.callback(*self.args, **self.kwargs)

    def _run(self):
        """
        Run desired callback and then reschedule Timer (if thread is not stopped)
        """
        try:
            self.run()
        except Exception as e:
            logging.exception("Exception in running periodic thread")
        finally:
            with self.schedule_lock:
                if not self.stop:
                    self.schedule_timer()

    def schedule_timer(self):
        """
        Schedules next Timer run
        """
        self.current_timer = threading.Timer(
            self.period, self._run, *self.args, **self.kwargs)
        if self.name:
            self.current_timer.name = self.name
        self.current_timer.start()

    def cancel(self):
        """
        Mimics Timer standard cancel method
        """
        with self.schedule_lock:
            self.stop = True
            if self.current_timer is not None:
                self.current_timer.cancel()

    def join(self):
        """
        Mimics Thread standard join method
        """
        self.current_timer.join()



import os
import sys
import logging
import argparse
from pilapse.server import start_server
from pilapse.image_saver import ImageSaver

logging.basicConfig(level=logging.INFO,
                    format='%(asctime)s %(name)-12s %(levelname)-8s %(message)s',
                    datefmt='%Y-%m-%d %H:%M:%S')


def run():
    import argparse
    parser = argparse.ArgumentParser()
    parser.add_argument("--port", default=8080, type=int,
                        help="HTTP server listen port")
    parser.add_argument("--path", default="./",
                        help="Directory to store images")
    parser.add_argument(
        "--name", default="img-%Y-%m-%d_%H%M%S.jpg", help="Image file name template")
    parser.add_argument("--period", default=60, type=int,
                        help="Timelapse frame period")
    parser.add_argument("--raspistill-args", default="-w 1920 -h 1080",
                        help="Extra arguments to raspistill")
    args = parser.parse_args()
    image_saver = ImageSaver(args)
    iss = image_saver.start()
    try:
        start_server(port=args.port, path=args.path)
    except KeyboardInterrupt as e:
        print("\nGot", type(e).__name__, 'exiting...')
        iss.cancel()
        try:
            sys.exit(0)
        except SystemExit:
            os._exit(0)


if __name__ == '__main__':
    run()



import logging
import subprocess
from datetime import datetime
from pathlib import Path
from pilapse.scheduler import PeriodicThread

logger = logging.getLogger('imageSaver')


class ImageSaver():
    def __init__(self, args):
        self.path = args.path
        self.name = args.name
        self.period = args.period
        self.raspistill_args = args.raspistill_args

    def save_image(self):
        t = datetime.now()
        path = Path(self.path).expanduser().joinpath(
            t.strftime(self.name)).absolute()
        cmd = ['raspistill', self.raspistill_args, '-o', str(path)]
        logger.debug(' '.join(cmd))
        try:
            out = subprocess.run(cmd, capture_output=True, text=True)
            if out.returncode != 0:
                logger.error("raspistill returned non-zero result")
                logger.error(out.stdout)
                logger.error(out.stderr)
        except Exception as e:
            logger.error('Cannot save file ({}): {}'.format(path, e))

    def start(self):
        logger.info('Saving images to {} with period of {}s'.format(
            self.path, self.period))
        pt = PeriodicThread(
            self.save_image, period=self.period, name='imageSaver')
        pt.start()
        return pt
